import { Component, OnInit, ElementRef, HostListener } from '@angular/core';
import { DataService } from '../../../services/data/data.service';
import { TranHistoryDetails } from '../../../classes/tran-history-details';
import Chart from 'chart.js/auto';

@Component({
	selector: 'dashboard',
	templateUrl: './dashboard.component.html',
	styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent implements OnInit {
	public selectedGrowthRange: string = 'Month';
	public ticklerList: TranHistoryDetails[];
	public sortedTicklerList: TranHistoryDetails[];
	public sortColumn: string;
	public loading: boolean = false;
	public sortDirection: number = 1;
	public sortDescription: string = '';
    private growthChart: any = null;
    private animateCharts: Boolean = true;
    private chartFontColor: String = '';
    private chartGridColor: String = '';
    private chartDataSeriesTranslucentColor1: String = '';
    private chartDataSeriesTranslucentColor2: String = '';

	constructor(
        private dataService: DataService,
        private elRef:ElementRef
    ) { }

	ngOnInit(): void {
		this.loading = true;
		this.dataService.GetTransactions().then(results => {
			this.loading = false;
			this.ticklerList = results;
			this.sortedTicklerList = this.ticklerList.slice();
			this.sortHistory('TranDate');
		});

        // Check if the user requests no animation
        const mediaQuery = window.matchMedia("(prefers-reduced-motion: reduce)");
        if (!mediaQuery || mediaQuery.matches) {
            this.animateCharts = false;
        }

        // Set up chart in page
        this.setUpCharts();
	}

    @HostListener('window:rui-load-theme', ['$event'])
    private themeChanged(e) {
        this.setUpCharts();
    }

	// Sort the tickler history
	public sortHistory(sortColumn: string): any {
		// Reverse the sort direction
		this.sortDirection *= -1;
		const sortDirection: number = this.sortDirection;

		// Remember the sort column
		this.sortColumn = sortColumn;

		this.sortedTicklerList.sort((itemA: TranHistoryDetails, itemB: TranHistoryDetails) => {
			// Compare ascendening
			let sortResult = 0;
			if (itemA[sortColumn] < itemB[sortColumn]) {
				sortResult = -1;
			} else if (itemA[sortColumn] > itemB[sortColumn]) {
				sortResult = 1;
			}

			// Adjust for sort direction
			sortResult *= sortDirection;

			return sortResult;
		});

		this.updateSortDescription();
	}

	private updateSortDescription(): void {
		this.sortDescription = 'Sorted by ';
		switch (this.sortColumn) {
			case 'TranDate': {
				this.sortDescription += 'Date';
				break;
			}
			case 'Payee': {
				this.sortDescription += 'Payee';
				break;
			}
		}
		this.sortDescription += (this.sortDirection === -1 ? ' (decreasing) ' : ' (increasing) ');
	}


    // Set up all charts in page
    private setUpCharts() {
        setTimeout(() => {
            // Grab chart colors from themed CSS vars
            this.getChartColors();

            // Set up each chart
            this.setUpGrowthChart();
        }, 100);
    }

    // Grab chart colors from themed CSS vars
    private getChartColors() {
        const styles = getComputedStyle(this.elRef.nativeElement);
        this.chartFontColor = String(styles.getPropertyValue('--text-regular'));
        this.chartGridColor = String(styles.getPropertyValue('--chart-grid-color'));
        this.chartDataSeriesTranslucentColor1 = String(styles.getPropertyValue('--chart-data-series-translucent-color-1'));
        this.chartDataSeriesTranslucentColor2 = String(styles.getPropertyValue('--chart-data-series-translucent-color-2'));
    }

    private setUpGrowthChart() {
        const canvas = document.querySelector('#growthChart');
        if (canvas) {
            if (this.growthChart) {
                // If chart already exists, update it with current theme's chart colors
                this.growthChart.data.datasets[0].backgroundColor = this.chartDataSeriesTranslucentColor1;
                this.growthChart.data.datasets[1].backgroundColor = this.chartDataSeriesTranslucentColor2;
                this.growthChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.growthChart.options.scales.x.ticks.color = this.chartFontColor;
                this.growthChart.options.scales.x.grid.color = this.chartGridColor;
                this.growthChart.options.scales.y.ticks.color = this.chartFontColor;
                this.growthChart.options.scales.y.grid.color = this.chartGridColor;
                this.growthChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.growthChart = new Chart(canvas as any, {
                    type: 'bar',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Primary',
                                backgroundColor: this.chartDataSeriesTranslucentColor1 as any,
                                data: [65, 59, 80, 81, 56, 55, 40]
                            },
                            {
                                label: 'Secondary',
                                backgroundColor: this.chartDataSeriesTranslucentColor2,
                                data: [28, 48, 40, 19, 86, 27, 90]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor as any
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            },
                            y: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            }
                        }
                    }
                });
            }
        }
    }

    public openPattern(url: string): void {
        const win = window.open(url, '_blank');
        win.focus();
    }

}
