import { Component, OnInit, ElementRef, HostListener } from '@angular/core';
import { Tickler } from '../../../classes/Tickler';
import { ServiceEventNearPastDate } from '../../../classes/ServiceEventNearPastDate';
import Chart from 'chart.js/auto';

@Component({
    selector: 'data-dense',
    templateUrl: './data-dense.component.html',
    styleUrls: ['./data-dense.component.scss']
})
export class DataDenseComponent implements OnInit {
    public eventsNearPastDate: ServiceEventNearPastDate[];
    public sortEventsColumn: string;
    public sortEventsDirection: number = 1;
    public ticklers: Tickler[];
    public sortTicklersColumn: string;
    public sortTicklersDirection: number = 1;
    private durationChart: any = null;
    private animateCharts: Boolean = true;
    private chartFontColor: String = '';
    private chartGridColor: String = '';
    private chartDataSeriesTranslucentColor1: String = '';
    private chartDataSeriesTranslucentColor2: String = '';
    
    constructor(private elRef:ElementRef) {}

    ngOnInit(): void {
        // Fake event data
        this.eventsNearPastDate = [
            new ServiceEventNearPastDate('Near', 'Elizabeth', 'Powell', 'Return call', new Date()),
            new ServiceEventNearPastDate('Near', 'Keith', 'Johnson', 'Status Report', new Date())
        ];
        this.sortEventsNearPast('NeededBy');

        // Fake tickler data
        this.ticklers = [
            new Tickler(new Date('6/11/2016'), 'Tamera Lynn Dowling', 'New Accounts'),
            new Tickler(new Date('3/2/2016'), 'Huda Mohammed Al-sultan', 'Account Test'),
            new Tickler(new Date('3/1/2016'), 'Wanda L Reno', 'First Phone Contact'),
            new Tickler(new Date('3/1/2016'), 'Samuel Adams', 'Client Contact'),
            new Tickler(new Date('3/1/2016'), 'Qiong Wang', 'Consolidation')
        ];
        this.sortTicklers('NeededBy');

        // Check if the user requests no animation
        const mediaQuery = window.matchMedia("(prefers-reduced-motion: reduce)");
        if (!mediaQuery || mediaQuery.matches) {
            this.animateCharts = false;
        }

        // Set up chart in page
        this.setUpCharts();
    }

    @HostListener('window:rui-load-theme', ['$event'])
    private themeChanged(e) {
        this.setUpCharts();
    }

    public openPattern(url: string): void {
        const win = window.open(url, '_blank');
        win.focus();
    }

    public sortEventsNearPast(sortColumn: string): any {
        // Reverse the sort direction
        this.sortEventsDirection *= -1;
        const sortDirection: number = this.sortEventsDirection;

        // Remember the sort column
        this.sortEventsColumn = sortColumn;

        this.eventsNearPastDate.sort((itemA: ServiceEventNearPastDate, itemB: ServiceEventNearPastDate) => {
            // Compare ascendening
            let sortResult = 0;
            if (itemA[sortColumn] < itemB[sortColumn]) {
                sortResult = -1;
            } else if (itemA[sortColumn] > itemB[sortColumn]) {
                sortResult = 1;
            }

            // Adjust for sort direction
            sortResult *= sortDirection;

            return sortResult;
        });
    }

    public sortTicklers(sortColumn: string): any {
        // Reverse the sort direction
        this.sortTicklersDirection *= -1;
        const sortDirection: number = this.sortTicklersDirection;

        // Remember the sort column
        this.sortTicklersColumn = sortColumn;

        this.ticklers.sort((itemA: Tickler, itemB: Tickler) => {
            // Compare ascendening
            let sortResult = 0;
            if (itemA[sortColumn] < itemB[sortColumn]) {
                sortResult = -1;
            } else if (itemA[sortColumn] > itemB[sortColumn]) {
                sortResult = 1;
            }

            // Adjust for sort direction
            sortResult *= sortDirection;

            return sortResult;
        });
    }

    // Set up all charts in page
    private setUpCharts() {
        setTimeout(() => {
            // Grab chart colors from themed CSS vars
            this.getChartColors();

            // Set up each chart
            this.setUpDurationChart();
        }, 100);
    }

    // Grab chart colors from themed CSS vars
    private getChartColors() {
        const styles = getComputedStyle(this.elRef.nativeElement);
        this.chartFontColor = String(styles.getPropertyValue('--text-regular'));
        this.chartGridColor = String(styles.getPropertyValue('--chart-grid-color'));
        this.chartDataSeriesTranslucentColor1 = String(styles.getPropertyValue('--chart-data-series-translucent-color-1'));
        this.chartDataSeriesTranslucentColor2 = String(styles.getPropertyValue('--chart-data-series-translucent-color-2'));
    }

    private setUpDurationChart() {
        const canvas = document.querySelector('#durationChart');
        if (canvas) {
            if (this.durationChart) {
                // If chart already exists, update it with current theme's chart colors
                this.durationChart.data.datasets[0].backgroundColor = this.chartDataSeriesTranslucentColor1;
                this.durationChart.data.datasets[1].backgroundColor = this.chartDataSeriesTranslucentColor2;
                this.durationChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.durationChart.options.scales.x.ticks.color = this.chartFontColor;
                this.durationChart.options.scales.x.grid.color = this.chartGridColor;
                this.durationChart.options.scales.y.ticks.color = this.chartFontColor;
                this.durationChart.options.scales.y.grid.color = this.chartGridColor;
                this.durationChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.durationChart = new Chart(canvas as any, {
                    type: 'bar',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Primary',
                                backgroundColor: this.chartDataSeriesTranslucentColor1 as any,
                                data: [65, 59, 80, 81, 56, 55, 40]
                            },
                            {
                                label: 'Secondary',
                                backgroundColor: this.chartDataSeriesTranslucentColor2,
                                data: [28, 48, 40, 19, 86, 27, 90]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor as any
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            },
                            y: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            }
                        }
                    }
                });
            }
        }
    }

}
