import { Component, OnInit, Output, EventEmitter, OnChanges, OnDestroy } from '@angular/core';
import { Subscription } from 'rxjs';
import { Address } from '../../classes/address';
import { AddressData } from '../../classes/addressData';

import { JhaDynamicComponentLoaderService } from '@jha/rui-angular/jha-responsive-core';

import { EditAddressComponent } from '../edit-address/edit-address.component';

@Component({
	selector: 'select-address',
	templateUrl: './select-address.component.html',
	styleUrls: ['./select-address.component.scss']
})
export class SelectAddressComponent implements OnInit, OnDestroy {
	public EditAddressComponentClass = EditAddressComponent;
	public addressList: Array<Address> = null;
	public addressData: AddressData = new AddressData(1, 1, []);
	public confirmDeleteAddress: boolean = false;
	public popupNotificationText: string = '';
	public showPopupNotification: boolean = false;
	public hasUnsavedChanges: boolean = false;
	private stackedViewDataSubscription: Subscription;
	private stackedViewActionSubscription: Subscription;

	constructor(
		private dynamicComponentLoaderService: JhaDynamicComponentLoaderService
	) {	}

	ngOnInit(): void {

		// Subscribe to stacked view data service
		this.stackedViewDataSubscription = this.dynamicComponentLoaderService.stackedViewData.subscribe(data => {
			if (data) {
				this.addressList = data.AddressList;
				this.addressData = new AddressData(data.SelectedAddressId, data.EditingAddressId, data.AddressList);
			}
		});

		// Subscribe to stacked view action service
		this.stackedViewActionSubscription = this.dynamicComponentLoaderService.stackedViewAction.subscribe(action => {
			if (action) {
				this.processMessage(action);
			}
		});

	}

	// Unsubscribe from subscription on destroy
	ngOnDestroy(): void {
		this.stackedViewDataSubscription.unsubscribe();
		this.stackedViewActionSubscription.unsubscribe();
	}

	// Listen for stacked views actions
	private processMessage(stackedViewAction: any): void {
		if (stackedViewAction.stackedViewPayload.stackedViewName === 'SelectAddressComponent') {
			if (stackedViewAction.stackedViewPayload.stackedViewAction === 'JHA_STACKEDVIEWS_UNSAVEDCHANGES') {
				// determine if there are unsaved changes
				stackedViewAction.stackedViewPayload.stackedViewCallback(this.hasUnsavedChanges);
			} else if (stackedViewAction.stackedViewPayload.stackedViewAction === 'JHA_STACKEDVIEWS_SAVEANDCLOSEVIEW') {
				this.removeStackedView(this.addressData);
				stackedViewAction.stackedViewPayload.stackedViewCallback(true);
			}
		}
	}

	// Select a new address, update address data and remove view
	public selectAddress(addressId: any): void {
		this.addressData.SelectedAddressId = addressId;
		this.removeStackedView(this.addressData);
	}

	// Stack the Edit Address view, passing the ID of the address to edit
	public editAddress(addressId: number): void {
		this.addressData.EditingAddressId = addressId;
		this.addStackedView(this.EditAddressComponentClass, 'EditAddressComponent', this.addressData);
	}

	// Prompt delete address confirmation
	public showDeleteAddressConfirmation(addressId: number): void {
		this.confirmDeleteAddress = true;
		this.addressData.SelectedAddressId = addressId;
	}

	// Delete Address from list
	public deleteAddress(): void {
		const index: number = this.addressData.AddressList.findIndex((address) => address.Id === this.addressData.SelectedAddressId);
		if (index >= 0) {
			this.addressData.AddressList.splice(index, 1);
			this.addressList = this.addressData.AddressList;
			this.popupNotificationText = 'Address deleted';
			this.showPopupNotification = true;
		}
		this.hasUnsavedChanges = true;
	}

	// Confirm address delete dialog
	public deleteAddressConfirmed(e): void {
		if (e.detail === 'Delete') {
			this.deleteAddress();
		}
	}

	// Close stacked view without saving data
	public closeSelectAddress(): void {
		this.removeStackedView();
	}

	// Add second level stacked view (Edit Address)
	private addStackedView(stackedComponent: any, stackedComponentName: string, data: any): void {
		if (stackedComponent) {
			this.dynamicComponentLoaderService.addStackedView(stackedComponent, stackedComponentName, data);
		}
	}

	// Once an address has been selected, call the service to remove the view and send the updated address
	private removeStackedView(address?: any): void {
		this.dynamicComponentLoaderService.removeStackedView('SelectAddressComponent', address);
	}

}
