import { Component, Input, HostListener, ElementRef, OnInit, OnDestroy, AfterViewInit, QueryList, ComponentFactoryResolver, ViewChildren } from '@angular/core';
import { Router, NavigationEnd, Event as NavigationEvent } from '@angular/router';
import { JhaDynamicComponentLoaderService, JhaDynamicComponentContainerDirective } from '@jha/rui-angular/jha-responsive-core';
import { ruiLoadTheme } from '@jha/rui-wc/js/rui-theming';

@Component({
    selector: 'jha-app',
    templateUrl: './jha-app.component.html',
    styleUrls: ['./jha-app.component.scss'],
    providers: [JhaDynamicComponentLoaderService]
})
export class JhaAppComponent implements OnInit, OnDestroy, AfterViewInit {
    @Input() routeMatchPrefix: string = '';
    @Input() themeFilePath: string = './dist/rui-themes';
    @Input() includeBannerNotification: boolean = true;
    @Input() includeSessionTimeout: boolean = true;
    @Input() includeUnsavedChangesDialog: boolean = true;
    private scrollContainer = null;
    public stackedViewLevel = 0;
    @ViewChildren(JhaDynamicComponentContainerDirective) containers!: QueryList<JhaDynamicComponentContainerDirective>;

    constructor(
        private router: Router,
        private elRef: ElementRef,
        private dynamicComponentLoaderService: JhaDynamicComponentLoaderService,
        private componentFactoryResolver: ComponentFactoryResolver
    ) {

        // Handler for when a route was successfully changed
        router.events.forEach((event: NavigationEvent) => {
            if (event instanceof NavigationEnd) {
                // Construct route entered custom event
                let routeEnteredEvent = new CustomEvent('rui-route-entered', {
                    detail: {
                        routeId: this.routeMatchPrefix + event.url
                    },
                    bubbles: true,
                    composed: true
                });

                // Fire after a short delay to allow for initial route load
                setTimeout(() => dispatchEvent(routeEnteredEvent), 10);

                // Scroll to the top of the page after route change
                this.performScrollToTop('immediate');
            }
        });
    }

    public ngOnInit() {
        // The scroll behavior here is required for several Responsive UI elements.
        this.scrollContainer = this.elRef.nativeElement.querySelector('#app-content');
        this.scrollContainer.addEventListener('scroll', (e) => this.scrolled(e));
    }

    public ngOnDestroy() {
        // The scroll behavior here is required for several Responsive UI elements.
        this.scrollContainer.removeEventListener('scroll', (e) => this.scrolled(e));
    }

    ngAfterViewInit(): void {
        this.dynamicComponentLoaderService.stackedViewLoader.subscribe(viewState => {
            if (viewState === 'remove') {
                this.removeStackedView();
            } else if (viewState === 'removeAll') {
                this.removeStackedView(true);
            } else {
                this.loadStackedView(viewState);
            }
        });
    }

    // Send rui-scrolled custom event when the main scroll region scrolls.
    private scrolled(e) {
        let scrolledEvent = new CustomEvent('rui-scrolled', {
            detail: {
                scrollTop: this.scrollContainer.scrollTop,
                scrollLeft: this.scrollContainer.scrollLeft
            },
            bubbles: true,
            composed: true
        }
        );
        dispatchEvent(scrolledEvent);
    }

    // Scroll the page to the top on request. The custom event below is sent
    // by the "back to top" button in rui-function-view but can also be
    // sent by the application.
    @HostListener('window:rui-scroll-to-top', ['$event'])
    private scrollToTop(e) {
        this.performScrollToTop(e.detail);
    }

    private performScrollToTop(scrollMode) {
        if (scrollMode === 'immediate') {
            // Immediate scroll
            this.scrollContainer.scrollTo(0, 0);
        } else {
            // Smooth scrolls
            this.scrollContainer.scrollTo({ top: 0, left: 0, behavior: 'smooth' });
        }
    }

    // Set proper CSS when function view is busy
    @HostListener('window:rui-function-busy', ['$event'])
    private functionBusy(e) {
        if (e.detail) {
            // Turn off scrolling when function is busy
            this.scrollContainer.classList = 'rui-app-content-no-scroll';
        } else {
            // Allow scrolling when function is not busy
            this.scrollContainer.classList = 'rui-app-content';
        }
    }

    // Display a banner notification when requested by the app
    @HostListener('window:rui-show-banner-notification', ['$event'])
    private displayBanner(e) {
        e.preventDefault();
        e.stopPropagation();

        try {
            // Parse banner details from event
            const bannerData = JSON.parse(e.detail);

            // Find banner in the this template
            const banner = this.elRef.nativeElement.querySelector('#banner');
            if (banner) {
                // Update its properties and show it
                banner.type = bannerData.bannerType;
                banner.innerHTML = bannerData.bannerText;
                banner.isVisible = true;
            }
        } catch (error) {
            console.log(`Error parsing banner display request: ${error}`)
        }
    }

    // Hide a banner notification when requested by the app
    @HostListener('window:rui-hide-banner-notification', ['$event'])
    private hideBanner(e) {
        e.preventDefault();
        e.stopPropagation();

        // Find banner in the this template
        const banner = this.elRef.nativeElement.querySelector('#banner');
        if (banner) {
            // Hide it
            banner.isVisible = false;
        }
    }

    // Load a new theme when requested by the app
    @HostListener('window:rui-load-theme', ['$event'])
    private themeChanged(e) {
        // Get the theme file from the event detail
        const themeFile = e.detail;

        // Load theme file from the specified path
        ruiLoadTheme(themeFile, this.themeFilePath);

        // Save the loaded theme file for future reference
        if (typeof (Storage) !== "undefined" && themeFile) {
            localStorage.setItem("rui-theme", themeFile);
        }
    }

    public loadStackedView(viewState: any): void {
        let viewContainerRef: any;

        if (this.stackedViewLevel < 2 && viewState) {
            this.stackedViewLevel++;
            viewContainerRef = this.containers.last.viewContainerRef;

            const componentFactory = this.componentFactoryResolver.resolveComponentFactory(viewState.componentClass);

            if (this.stackedViewLevel === 1) {
                viewContainerRef = this.containers.last.viewContainerRef;
            } else if (this.stackedViewLevel === 2) {
                viewContainerRef = this.containers.first.viewContainerRef;
            }
            viewContainerRef.clear();

            // Load new component inside of view container
            const component = viewContainerRef.createComponent(componentFactory);
        }
    }

    public removeStackedView(removeAll?: boolean): void {
        if (removeAll) {
            this.containers.last.viewContainerRef.clear();
            this.containers.first.viewContainerRef.clear();
            this.stackedViewLevel = 0;
        } else {
            if (this.stackedViewLevel === 1) {
                this.containers.last.viewContainerRef.clear();
            } else if (this.stackedViewLevel === 2) {
                this.containers.first.viewContainerRef.clear();
            }

            if (this.stackedViewLevel !== 0) {
                this.stackedViewLevel--;
            }
        }
    }

}
