﻿import { Injectable } from '@angular/core';

import { CustomerDetails } from '../../classes/customer-details';
import { JhaListOption } from '@jha/rui-angular/jha-responsive-core';
import { Alert } from '../../classes/alert';
import { AlertGroup } from '../../classes/alert-group';
import { Action } from '../../classes/action';
import { Email } from '../../classes/email';
import { AccountFull } from '../../classes/account-full';
import { MailingAddress } from '../../classes/mailing-address';

import { SelectItem } from 'primeng/api';
import { Observable, of } from 'rxjs';

@Injectable()
export class CustomerDetailsService {
	public getCustomerDetailInfo(): Promise<CustomerDetails> {
		// Dummy up customer detail info here
		const customerDetailInfo = new CustomerDetails(
			'Nancy Davolio',
			true,
			8.75,
			111223333,
			new Date(1976, 3, 1),
			0,
			45000.00,
			false,
			['Checking', 'Money Market', 'CD'],
			'Silver',
			'Customer Account',
			new Date(2016, 0, 15, 14, 18),
			'123 Main St',
			'Springfield',
			'MO',
			658071234,
			'Collin',
			'USA',
			'Domestic',
			'417-888-9631',
			'417-541-1980',
			'417-626-8827',
			'carrie.smith@gmail.com',
			['security-other', 'check-balancing'],
			'',
			{},
			[]
		);

		// Recalculate the age from the birthdate
		let calculatedAge = 0;
		if (customerDetailInfo.birthdate != null) {
			const fieldDate = customerDetailInfo.birthdate;
			const today = new Date();
			calculatedAge = today.getFullYear() - fieldDate.getFullYear();
			if (today.getMonth() < fieldDate.getMonth() || (today.getMonth() === fieldDate.getMonth() && today.getDate() < fieldDate.getDate())) {
				calculatedAge--;
			}
		}
		customerDetailInfo.age = calculatedAge;

		return new Promise((resolve) => {
			setTimeout(() => resolve(customerDetailInfo), 500);
		});
	}

	// Post customer detail info
	public postCustomerDetailInfo(customerInfo: CustomerDetails): Promise<boolean> {
		customerInfo.recordLevelValidationErrors = [];
		customerInfo.fieldLevelValidationErrors = {};

		// Fake record-level validations
		if ((customerInfo.country === 'USA' && customerInfo.addressType === 'International') ||
			(customerInfo.country !== 'USA' && customerInfo.addressType === 'Domestic')) {
			customerInfo.recordLevelValidationErrors.push(customerInfo.country + ' is an invalid country for ' + customerInfo.addressType + ' address type');
		}

		// Fake field-level validations
		this.fakeValidateCustomerDetailField('Name', customerInfo);
		this.fakeValidateCustomerDetailField('InterestRate', customerInfo);

		// Return value is an error flag, so check for errors
		let errorFlag: boolean = false;
		if (customerInfo.recordLevelValidationErrors.length > 0 ||
			Object.keys(customerInfo.fieldLevelValidationErrors).length > 0) {
			errorFlag = true;
		}

		return new Promise((resolve) => {
			setTimeout(() => resolve(errorFlag), 500);
		});
	}

	// Fake field-level validation; these would typically be done by the back-end service
	public fakeValidateCustomerDetailField(fieldId: string, customerInfo: CustomerDetails) {
		let validationError = '';

		if (fieldId === 'Name') {
			if (!customerInfo.name) {
				validationError = 'Please fill out this field.';
			} else if (customerInfo.name && customerInfo.name.toLowerCase() === 'jimbo') {
				validationError = 'This name is not allowed.';
			}
		} else if (fieldId === 'InterestRate') {
			if (!Boolean(customerInfo.interestRate % 2)) {
				validationError = 'Value must be odd.';
			}
		}

		if (validationError) {
			customerInfo.fieldLevelValidationErrors[fieldId] = validationError;
		}
	}

	public getAlertList(): Promise<Alert[]> {
		// Dummy up alert list here
		const alertList = new Array<Alert>();
		alertList.push(new Alert('Lost/Stolen', 'ATM card is lost or stolen'));

		return new Promise((resolve) => {
			setTimeout(() => resolve(alertList), 500);
		});
	}

	public getActionList(): Promise<Action[]> {
		// Dummy up action list here
		const actionList = new Array<Action>();
		actionList.push(new Action('Bad Address', 'No valid address for customer', '(1 of 3)', 'Add'));
		actionList.push(new Action('E-mail', 'No e-mail on file', '(2 of 3)', 'Add'));
		actionList.push(new Action('Photo ID', 'No ID images available for customer', '(3 of 3)', 'Add'));

		return new Promise((resolve) => {
			setTimeout(() => resolve(actionList), 500);
		});
	}

	public getEmailList(): Promise<Email[]> {
		// Dummy up email list here
		const emailList = new Array<Email>();
		emailList.push(new Email('carrie.smith@gmail.com', 'Edit'));
		emailList.push(new Email('csmith@jonestyler.com', 'Edit'));

		return new Promise((resolve) => {
			setTimeout(() => resolve(emailList), 500);
		});
	}

	// Get accounts
	public getAccountList(): Promise<AccountFull[]> {
		// Dummy up account list here
		const accountList = new Array<AccountFull>();
		accountList.push(new AccountFull('Web', '788300000347', 'NetTeller', 'Demand Deposit', '', null, '(1 of 3)', true, 'Details'));
		accountList.push(new AccountFull('Check', '53041', 'T1 REG DDA', 'Demand Deposit', 'Escheat', 44.99, '(2 of 3)', true, 'Details'));
		accountList.push(new AccountFull('NonJHAMisc', '123456', 'Shareholder', 'Savings', '', null, '(3 of 3)', false, ''));

		return new Promise((resolve) => {
			setTimeout(() => resolve(accountList), 500);
		});
	}

	// Get mailing addresses
	public getMailingAddressList(): Promise<MailingAddress[]> {
		// Dummy up account list here
		const alertList = new Array<MailingAddress>();
		alertList.push(new MailingAddress('Physical', 'Domestic', '123 Main St', '', 'Monett', 'MO', '65708', '', '', null, null, 'Yes'));
		alertList.push(new MailingAddress('Seasonal', 'Domestic', '456 Sunshine Lane', '', 'Miami', 'FL', '12345', '', '', new Date('10/1/15'), new Date('12/31/15'), 'Yes'));

		return new Promise((resolve) => {
			setTimeout(() => resolve(alertList), 500);
		});
	}

	public getAccountTypeList(): SelectItem[] {
		const accountTypeList =
		[
			{label: 'Checking', value: 'Checking'},
			{label: 'Business Checking', value: 'Business Checking'},
			{label: 'Student Checking', value: 'Student Checking'},
			{label: 'Savings', value: 'Savings'},
			{label: 'Money Market', value: 'Money Market'},
			{label: 'CD', value: 'CD'},
			{label: 'IRA', value: 'IRA'}
		];
		return accountTypeList;
	}

	public getStateList(): JhaListOption[] {
		const stateList =
		[
			new JhaListOption('Alabama', 'AL'),
			new JhaListOption('Alaska', 'AK'),
			new JhaListOption('Arizona', 'AZ'),
			new JhaListOption('Arkansas', 'AR'),
			new JhaListOption('California', 'CA'),
			new JhaListOption('Colorado', 'CO'),
			new JhaListOption('Connecticut', 'CT'),
			new JhaListOption('Delaware', 'DE'),
			new JhaListOption('Florida', 'FL'),
			new JhaListOption('Georgia', 'GA'),
			new JhaListOption('Hawaii', 'HI'),
			new JhaListOption('Idaho', 'ID'),
			new JhaListOption('Illinois', 'IL'),
			new JhaListOption('Indiana', 'IN'),
			new JhaListOption('Iowa', 'IA'),
			new JhaListOption('Kansas', 'KS'),
			new JhaListOption('Kentucky', 'KY'),
			new JhaListOption('Louisiana', 'LA'),
			new JhaListOption('Maine', 'ME'),
			new JhaListOption('Maryland', 'MD'),
			new JhaListOption('Massachusetts', 'MA'),
			new JhaListOption('Michigan', 'MI'),
			new JhaListOption('Minnesota', 'MN'),
			new JhaListOption('Mississippi', 'MS'),
			new JhaListOption('Missouri', 'MO'),
			new JhaListOption('Montana', 'MT'),
			new JhaListOption('Nebraska', 'NE'),
			new JhaListOption('Nevada', 'NV'),
			new JhaListOption('New Hampshire', 'NH'),
			new JhaListOption('New Jersey', 'NJ'),
			new JhaListOption('New Mexico', 'NM'),
			new JhaListOption('New York', 'NY'),
			new JhaListOption('North Carolina', 'NC'),
			new JhaListOption('North Dakota', 'ND'),
			new JhaListOption('Ohio', 'OH'),
			new JhaListOption('Oklahoma', 'OK'),
			new JhaListOption('Oregon', 'OR'),
			new JhaListOption('Pennsylvania', 'PA'),
			new JhaListOption('Rhode Island', 'RI'),
			new JhaListOption('South Carolina', 'SC'),
			new JhaListOption('South Dakota', 'SD'),
			new JhaListOption('Tennessee', 'TN'),
			new JhaListOption('Texas', 'TX'),
			new JhaListOption('Utah', 'UT'),
			new JhaListOption('Vermont', 'VT'),
			new JhaListOption('Virginia', 'VA'),
			new JhaListOption('Washington', 'WA'),
			new JhaListOption('West Virginia', 'WV'),
			new JhaListOption('Wisconsin', 'WI'),
			new JhaListOption('Wyoming', 'WY')
		];
		return stateList;
	}

	public getAlertTypeList(): AlertGroup[] {
		const alertTypeGroups: AlertGroup[] = [
			{
				name: 'Security',
				alert: [
					{category: 'security-alerts', description: 'Security Alerts'},
					{category: 'security-other', description: 'Other Alerts'}
				]
			},
			{
				name: 'Checking Alerts',
				alert: [
					{category: 'check-balancing', description: 'Balancing & Spending'},
					{category: 'check-payments', description: 'Payments'},
					{category: 'check-security', description: 'Security Alerts'}
				]
			},
			{
				name: 'Savings Alerts',
				alert: [
					{category: 'savings-balancing', description: 'Balancing & Spending'},
					{category: 'savings-payments', description: 'Payments'},
					{category: 'savings-security', description: 'Security Alerts'}
				]
			},
			{
				name: 'Retirement Alerts',
				disabled: true,
				alert: [
					{category: 'retirement-balancing', description: 'Balancing & Spending', disabled: true},
					{category: 'retirement-payments', description: 'Payments', disabled: true},
					{category: 'retirement-security', description: 'Security Alerts', disabled: true}
				]
			},
			{
				name: 'Delivery Preferences',
				alert: [
					{category: 'delivery-email', description: 'Email'},
					{category: 'delivery-text', description: 'Text Message'},
					{category: 'delivery-phone', description: 'Phone Call'}
				]
			}
		];
		return alertTypeGroups;
	}

	public getCountryList(): string[] {
		const countryList =
		[
			'Afghanistan',
			'Albania',
			'Algeria',
			'Andorra',
			'Angola',
			'Antigua and Barbuda',
			'Argentina',
			'Armenia',
			'Australia',
			'Austria',
			'Azerbaijan',
			'Bahamas',
			'Bahrain',
			'Bangladesh',
			'Barbados',
			'Belarus',
			'Belgium',
			'Belize',
			'Benin',
			'Bhutan',
			'Bolivia',
			'Bosnia and Herzegovina',
			'Botswana',
			'Brazil',
			'Brunei',
			'Bulgaria',
			'Burkina Faso',
			'Burundi',
			'Cabo Verde',
			'Cambodia',
			'Cameroon',
			'Canada',
			'Central African Republic',
			'Chad',
			'Chile',
			'China',
			'Colombia',
			'Comoros',
			'Republic of the Congo',
			'Democratic Republic of theCongo',
			'Costa Rica',
			'Cote d\'Ivoire',
			'Croatia',
			'Cuba',
			'Cyprus',
			'Czech Republic',
			'Denmark',
			'Djibouti',
			'Dominica',
			'Dominican Republic',
			'Ecuador',
			'Egypt',
			'El Salvador',
			'Equatorial Guinea',
			'Eritrea',
			'Estonia',
			'Ethiopia',
			'Fiji',
			'Finland',
			'France',
			'Gabon',
			'Gambia',
			'Georgia',
			'Germany',
			'Ghana',
			'Greece',
			'Grenada',
			'Guatemala',
			'Guinea',
			'Guinea-Bissau',
			'Guyana',
			'Haiti',
			'Honduras',
			'Hungary',
			'Iceland',
			'India',
			'Indonesia',
			'Iran',
			'Iraq',
			'Ireland',
			'Israel',
			'Italy',
			'Jamaica',
			'Japan',
			'Jordan',
			'Kazakhstan',
			'Kenya',
			'Kiribati',
			'Kosovo',
			'Kuwait',
			'Kyrgyzstan',
			'Laos',
			'Latvia',
			'Lebanon',
			'Lesotho',
			'Liberia',
			'Libya',
			'Liechtenstein',
			'Lithuania',
			'Luxembourg',
			'Macedonia',
			'Madagascar',
			'Malawi',
			'Malaysia',
			'Maldives',
			'Mali',
			'Malta',
			'Marshall Islands',
			'Mauritania',
			'Mauritius',
			'Mexico',
			'Micronesia',
			'Moldova',
			'Monaco',
			'Mongolia',
			'Montenegro',
			'Morocco',
			'Mozambique',
			'Myanmar',
			'Namibia',
			'Nauru',
			'Nepal',
			'Netherlands',
			'New Zealand',
			'Nicaragua',
			'Niger',
			'Nigeria',
			'North Korea',
			'Norway',
			'Oman',
			'Pakistan',
			'Palau',
			'Palestine',
			'Panama',
			'Papua New Guinea',
			'Paraguay',
			'Peru',
			'Philippines',
			'Poland',
			'Portugal',
			'Qatar',
			'Romania',
			'Russia',
			'Rwanda',
			'St. Kitts and Nevis',
			'St. Lucia',
			'St. Vincent and The Grenadines',
			'Samoa',
			'San Marino',
			'Sao Tome and Principe',
			'Saudi Arabia',
			'Senegal',
			'Serbia',
			'Seychelles',
			'Sierra Leone',
			'Singapore',
			'Slovakia',
			'Slovenia',
			'Solomon Islands',
			'Somalia',
			'South Africa',
			'South Korea',
			'South Sudan',
			'Spain',
			'Sri Lanka',
			'Sudan',
			'Suriname',
			'Swaziland',
			'Sweden',
			'Switzerland',
			'Syria',
			'Taiwan',
			'Tajikistan',
			'Tanzania',
			'Thailand',
			'Timor-Leste',
			'Togo',
			'Tonga',
			'Trinidad and Tobago',
			'Tunisia',
			'Turkey',
			'Turkmenistan',
			'Tuvalu',
			'Uganda',
			'Ukraine',
			'United Arab Emirates',
			'UK',
			'USA',
			'Uruguay',
			'Uzbekistan',
			'Vanuatu',
			'Vatican City',
			'Venezuela',
			'Vietnam',
			'Yemen',
			'Zambia',
			'Zimbabwe'
		];

		return countryList;
	}

	// Mock file upload service
	public postFile(filesToUpload: any[]): Observable<any> {
		console.log(filesToUpload);
		return of(filesToUpload);
	}
}
